#define DEBUG 1
#include <Debug.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <Application.h>
#include <Roster.h>
#include <Alert.h>
#include <String.h>
#include <Path.h>
#include <Entry.h>
#include <FindDirectory.h>
#include <Directory.h>
#include <Node.h>
#include <fs_attr.h>

#include "ZKApp.h"
#include "ZKWindow.h"

ZooKeeperApp::ZooKeeperApp	(const char * a_app_sig)
 : BApplication	(a_app_sig)	
{
	argv_received	=	false;
	refs_received	=	false;
	
	if (CreateTempFolder() != B_OK)
		exit(-1);	

	m_window = 	new ZKWindow (BRect(300,200,495,320));
	
	GetPreferences();
}

ZooKeeperApp::~ZooKeeperApp	(void)	
{
	// void
}

void 
ZooKeeperApp::ArgvReceived	(int32 argc, char **argv)
{
	argv_received	=	true;

	BMessage *	msg = new BMessage(B_REFS_RECEIVED);
	int32	result;

	if(argc <= 1)	// no arguments
		return;

//	get current path

	char *	buffer = new char[4096];
	getcwd(buffer, 4096);
	BString	currentdir = buffer;
	if (currentdir.FindLast('/') != currentdir.Length()-1)
		currentdir.Append("/");
	delete buffer;

//	convert arguments to entry_refs

	for(int32 i = 1; i < argc; i++)
	{
		BString path_str = argv[i];
		result =  path_str.FindFirst("/");
		
		if (result == B_ERROR || result != 0)
		{
			path_str.Prepend(currentdir);
		}
			
		BEntry		entry(path_str.String());
		entry_ref	ref;
		entry.GetRef(& ref);
		
		msg->AddRef("refs", & ref);

	}
	
//	Send to RefsReceived
	RefsReceived(msg);
	delete msg;
}

void 
ZooKeeperApp::RefsReceived	(BMessage * a_message)
{
	refs_received	=	true;
	
	ProcessRefs(a_message);
}

void
ZooKeeperApp::ReadyToRun	(void)
{
	if (!argv_received && !refs_received)
	{
		m_window->Show();
		return;
	}

	PostMessage(B_QUIT_REQUESTED);
}

void 
ZooKeeperApp::MessageReceived	(BMessage * a_message)
{
	switch(a_message->what)
	{
		case ZOO_HELP_REQUESTED:
				Help();
				break;
				
		default:
				BApplication::MessageReceived(a_message);
				break;
	}
}

void
ZooKeeperApp::AboutRequested	(void)
{
	BAlert * alert = new BAlert("ZooKeeper", "ZooKeeper 1.0\n\n"
	"Freeware   " B_UTF8_COPYRIGHT " Jonas Sundström, Kirilla.com\n", 
	"Help", "Kirilla", "Ok", B_WIDTH_AS_USUAL, B_INFO_ALERT);

	int32 reply = alert->Go();
	
	if (reply == 1)
	{
		BMessage msg(B_ARGV_RECEIVED); 
		msg.AddString("argv", "app"); 
		msg.AddString("argv", "http://www.kirilla.com/");  
		msg.AddInt32("argc", 2); 
		be_roster->Launch ("application/x-vnd.Be.URL.http", &msg ); 			
	}
	
	if (reply == 0)
		Help();
}

void 
ZooKeeperApp::Help	(void)
{
	(new BAlert("ZooKeeper", 
	
	"ZooKeeper is an application that lets you specify "
	"a shell command to be executed on the files you drop on it.\n\n"
	
	"You can use ZooKeeper in a couple of ways:\n"
	"* Drop files on the icon of the executable\n"
	"* Select ZooKeeper from the Open With... -menu\n"
	"* Set your files to open with ZooKeeper\n\n"
	
	"When opened without files, ZooKeeper will let you edit its preferences.\n\n"
	
	"If you want ZooKeeper to do anything useful at all you "
	"need to supply a Command, including the $files variable, "
	"like this:\n/boot/beos/apps/StyledEdit $files\n"
	"(launches the files in StyledEdit)\n\n"
	"I'm sure you can come up with a more intelligent use for ZooKeeper. "
	"Perhaps you have a shell-script or some command-line application "
	"that you'd like ZooKeeper to \"front\".\n\n"
	
	"You don't need to supply a directory,\n"
	"it will default to your home folder.\n\n"
	
	"Your preferences are stored in the application file itself, "
	"to make it possible for you "
	"to have multiple ZooKeepers for different purposes. "
	"I advise you to give each ZooKeeper application file "
	"a unique name, describing its purpose, "
	"and a unique Application Signature.", "Right", 
	NULL, NULL, B_WIDTH_AS_USUAL, B_IDEA_ALERT))->Go();
}

void 
ZooKeeperApp::ProcessRefs	(BMessage * a_message)
{
	entry_ref	ref;
	int32		ref_count	=	0;

	BPath	path;
	BString	temp;
	BString	files;
		
	while(a_message->FindRef("refs", ref_count, &ref) == B_OK)
	{
		path.SetTo(& ref);		
		temp	=	path.Path();
		NameSafe( & temp);
		files << " " << temp;
		ref_count++;	
	}

	m_command.ReplaceAll("$files", files.String());

	RunScript();
}

void 
ZooKeeperApp::GetPreferences(void)	
{
	// read preferences from ZooKeeper app

	// app ref
	app_info	my_app_info;
	be_app->GetAppInfo(& my_app_info);
	m_app_ref	=	my_app_info.ref;

	// app node
	BNode	node	(& m_app_ref);
	
	char *	buffer	=	new char [4096];
	off_t	bytes	=	0;

	// dir
	bytes = node.ReadAttr("zook:dir", B_STRING_TYPE, 0, buffer, 4095);
	if (bytes >= 0)
	{
		buffer[bytes] = '\0';
		m_dir	=	buffer;
	}
	
	// command
	bytes = node.ReadAttr("zook:command", B_STRING_TYPE, 0, buffer, 4095);
	if (bytes >= 0)
	{
		buffer[bytes] = '\0';
		m_command	=	buffer;
	}
	
	// in terminal
	attr_info	info;
	if (node.GetAttrInfo("zook:terminal", & info) == B_OK)
		m_in_terminal	=	true;
	else
		m_in_terminal	=	false;
	
	// keep open
	if (node.GetAttrInfo("zook:keepopen", & info) == B_OK)
		m_keep_open	=	true;
	else
		m_keep_open	=	false;
	
	delete [] buffer;
}

void 
ZooKeeperApp::RunScript(void)
{
	// create kickstart_script
	BString kickstart_script	("#!/bin/sh\n");

	// working dir
	kickstart_script.Append("cd ");	
	kickstart_script.Append(m_dir);	
	kickstart_script.Append("\n");	

	// command
	kickstart_script.Append(m_command);
	kickstart_script.Append("\n");
	
	// keep open
	if (m_keep_open && m_in_terminal)
		kickstart_script.Append("read -p \"\n(Press Enter to close)\"\n");
	
	//	create tempfile name
	int32	counter	=	1;
	BString tempfile_name;
	tempfile_name << counter;
	while(m_tmp_dir->Contains(tempfile_name.String()))
	{
		counter	++;
		tempfile_name.Truncate(0);
		tempfile_name << counter;
	}
	
	// write kickstart script to file
	BFile	tmp_file;
	if (m_tmp_dir->CreateFile(tempfile_name.String(), & tmp_file) != B_OK)
		exit(-1);
		
	tmp_file.Write(kickstart_script.String(), kickstart_script.Length());
	
	// open Terminal with kickstart script
	
	BPath	tempfile_path	(m_tmp_dir, tempfile_name.String());
	BString command_string	(tempfile_path.Path());

	if (m_in_terminal)
	{
		command_string.Prepend(" ");
		command_string.Prepend(m_app_ref.name);
		command_string.Prepend("/boot/beos/apps/Terminal -t ");
	}
	
	command_string.Append(" &");
	
	PRINT(("system():\n%s\n", command_string.String()));
	system(command_string.String());	
}

void
ZooKeeperApp::NameSafe	(BString * a_string)
{
	int32	position = 0;
	while(position < a_string->	Length())
	{
		if      (a_string-> ByteAt (position) == '\"') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '`') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '$') { a_string-> Insert ("\\",position); position++; }
		position++;
	}
	
	a_string->Prepend("\"");
	a_string->Append("\"");
	
}

void
ZooKeeperApp::FullQuote	(BString * a_string)
{
	int32	position = 0;
	while(position < a_string->	Length())
	{
		if      (a_string-> ByteAt (position) == ';') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '&') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '(') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == ')') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '|') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '<') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '>') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '*') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '?') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '[') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == ']') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '~') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '@') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '!') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '"') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '\'') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '\\') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '`') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '$') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == ' ') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '{') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '}') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '#') { a_string-> Insert ("\\",position); position++; }
		position++;
	}
}

status_t
ZooKeeperApp::CreateTempFolder	(void)
{
	status_t status;
	BPath p;
	if (find_directory(B_COMMON_TEMP_DIRECTORY, &p) != B_OK)	return B_ERROR;
	
	m_tmp_dir	=	new BDirectory();
	BDirectory temp_dir;
	
	if ((status = m_tmp_dir->	SetTo(p.Path())) != B_OK) 	return status;
	status	=	m_tmp_dir->	CreateDirectory("Kirilla_ZooKeeper", &temp_dir);
	if (status != B_OK && status != B_FILE_EXISTS)	return status;

	p.SetTo(p.Path(),"Kirilla_ZooKeeper");
	if ((status = m_tmp_dir->	SetTo(p.Path())) != B_OK)	return status;
		
	BEntry entry(p.Path());
	if (entry.Exists()	&&	entry.IsDirectory())	return B_OK;
	else											return B_ERROR;
}
